using System;
using System.IO;

namespace SmileProxy
{
	public class CachedStream : System.IO.Stream
	{
		private bool			_CacheExists = false;
		private string			_CacheDir = "cache";
		private string			_Id;
		private Stream			_InStream  = null;
		private Stream			_CacheStream = null;

		public CachedStream(string id) : this(id, null) {
		}

		public CachedStream(string id, Stream inStream) {
			_Id = id;
			_InStream  = inStream;

			string cachePathTmp = this.CachePathTmp;
			if (System.IO.Directory.Exists(cachePathTmp)) {
				_CacheStream = null;
				_CacheExists = false;
				return;
			}
			System.IO.Directory.CreateDirectory(cachePathTmp);

			string cachePath    = this.CachePath;
			if (System.IO.File.Exists(cachePath)) {
				_CacheExists = true;
			} else {
				_CacheExists = false;
			}
			_CacheStream = new FileStream(cachePath, FileMode.OpenOrCreate, FileAccess.ReadWrite);
		}

		public string CachePath {
			get {
				return _CacheDir + "/" + _Id;
			}
		}

		public string CachePathTmp {
			get {
				return _CacheDir + "/" + _Id + ".lock";
			}
		}

		public bool CacheExists {
			get {
				return _CacheExists;
			}
		}

		public override bool CanRead {
			get {
				if (_InStream != null) {
					return _InStream.CanRead;
				}
				if (_CacheStream != null) {
					return _CacheStream.CanRead;
				}
				return false;
			}
		}

		public override bool CanSeek {
			get {
				if (_InStream != null) {
					return _InStream.CanSeek;
				}
				if (_CacheStream != null) {
					return _CacheStream.CanSeek;
				}
				return false;
			}
		}

		public override bool CanWrite {
			get {
				if (_InStream != null) {
					return _InStream.CanWrite;
				}
				if (_CacheStream != null) {
					return _CacheStream.CanWrite;
				}
				return false;
			}
		}

		public override void Close() {
			if (_CacheStream != null) {
				_CacheStream.Close();
				_CacheStream = null;
			}
			
			if (_InStream != null) {
				//_InStream.Close();
				_InStream = null;
			}

			this._CacheExists = false;
			try {
				System.IO.Directory.Delete(this.CachePathTmp, false);
			}
			catch {
			}

			base.Close();
		}

		public override void Flush() {
			if (_InStream != null) {
				_InStream.Flush();
			}
			if (_CacheStream != null) {
				_CacheStream.Flush();
			}
		}

		public override long Length {
			get {
				if (_InStream != null) {
					return _InStream.Length;
				}
				if (_CacheStream != null) {
					return _CacheStream.Length;
				}
				return 0;
			}
		}

		public override long Position {
			get {
				if (_InStream != null) {
					return _InStream.Position;
				}
				if (_CacheStream != null) {
					return _CacheStream.Position;
				}
				return 0;
			}
			set {
				if (_InStream != null) {
					_InStream.Position = value;
				}
				if (_CacheStream != null) {
					_CacheStream.Position = value;
				}
			}
		}

		public override void SetLength(long value) {
			if (_InStream != null) {
				_InStream.Position = value;
			}
			if (_CacheStream != null) {
				_CacheStream.Position = value;
			}			
		}

		public override long Seek(long offset, SeekOrigin origin) {
			long pos = 0;
			if (_InStream != null) {
				pos = _InStream.Seek(offset, origin);
			}
			if (_CacheStream != null) {
				pos = _CacheStream.Seek(offset, origin);
			}
			return pos;
		}

		public override int Read(byte[] buffer, int offset, int count) {
			int  len = 0;
			long pos = 0;
			if (this.CacheExists && _CacheStream != null) {
				len = _CacheStream.Read(buffer, offset, count);
				if (_InStream.CanSeek && _CacheStream.CanSeek) {
					pos = _CacheStream.Position;
				}
				if (_InStream != null) {
					if (_InStream.CanSeek && _CacheStream.CanSeek) {
						_InStream.Position = pos;
					}
				}
			}
			else if (_InStream != null){
				if (_InStream.CanSeek && _CacheStream.CanSeek) {
					pos = _InStream.Position;
				}
				len = _InStream.Read(buffer, offset, count);
				if (_CacheStream != null) {
					if (_InStream.CanSeek && _CacheStream.CanSeek) {
						_CacheStream.Position = pos;
					}
					_CacheStream.Write(buffer, offset, count);
				}
			}
			return len;
		}

		public override void Write(byte[] buffer, int offset, int count) {
			if (_InStream != null) {
				if (_InStream.CanSeek && _CacheStream.CanSeek) {
					_CacheStream.Position = _InStream.Position;
				}
				_InStream   .Write(buffer, offset, count);
			}
			if (_CacheStream != null) {
				_CacheStream.Write(buffer, offset, count);
			}
		}
	}
}
